/*
 Replication code for the Dynare WP 83
 "Policy Mix in an Oil Exporting Country: 
 Effectiveness of Countercyclical Measures in Mitigating External Shocks"
 (Diaf & Zakane) 
 based on the paper of Algozhina (2016).
 */
 

 
%=========================================================================%
% [A]. DECLARATION OF VARIABLES 
%=========================================================================%

var y yo yno c n kno ko gc kg infl ir rer fxr nx bf b of po inff cf iff fdi ph inv gi tax pg inflh lagr;

varexo epo ecf;

parameters iss betaf alpha alphao alphaf rhof kappa rhofdi div tauo psi delta deltag sigma theta thetaf gamma gamma2 eps phi phif mu phiinf phiy phiinff phiyf rho_ir rho_po phigi phigc phib phiof gamofgi gamofgc by gcy giy cy nxy bfy fdiy ofy so omega rhogc rhogi rhouf gamgc gamgi rhofxr fxry bss yss yoss koss gcss ofss tss css kgss nss knoss vgi vgc a1 a2;

iss=0.023; 
betaf=0.99; 
alpha=0.3; 
alphao=0.7; 
alphaf=0.32;
rhof=0.747;  
rhofdi=0.3;
div=0.05;
kappa=20;
tauo=0.27;
psi=0.16;  
delta=0.025; 
deltag=0.02; 
sigma=2; 
theta=0.9;  
thetaf=0.75;
gamma=0.68; 
gamma2=0.9;
phi=1.45;  
phif=1.45; 
mu=0.5;
phiinf=1;
phiy=0.6;  
phiinff=0.3; 
phiyf=0.125; 
rho_ir=0.05; 
rho_po=0.80; 
phigi=0.2;
phigc=0.95; 
phib=0.4; 

phiof=-0.3;   
gamofgi=0.1;   
gamofgc=0.2;   

gcy=0.08;
giy=0.07;
cy=0.61; 
nxy=0.2;
ofy=0.65;
bfy=2.17;
fdiy=0.05;
by=0.05;
so=0.4;
fxry=0.52; 
omega=0.54;
rhogc=0.44; 
rhogi=0.44; 
rhouf=0.8; 
a1=-1.4; 
a2=-0.1; 
gamgc=0.3;   
gamgi=0.3;  
vgi=-0.4; 
vgc=-0.4;
eps=9; 
rhofxr=0.7; 

% steady state equations

kgss=1678.1;  
yss=479.4565;
nss=(1-alpha)^(1/(phi-1))*(kgss^psi*alpha^alpha/((iss+delta-omega*(betaf-1/(1+iss))*(1+iss))^alpha*eps/(eps-1)))^(1/((1-alpha)*(phi-1)));
koss=((iss+delta)/(alphao*(1-tauo)))^(1/(alphao-1));
yoss=koss^alphao;
knoss=((yss-yoss)/(nss^(1-alpha)*kgss^psi))^(1/alpha);
ofss=(tauo+div*(1-tauo))*yoss/(1-rhof/betaf);
bss=(yoss^vgi*((1-rhof)*ofss/betaf)^gamofgi/(deltag*kgss))^(1/gamgi);
gcss=yoss^vgc*((1-rhof)*ofss/betaf)^gamofgc/bss^gamgc;
css=yss-(1-mu)*delta*knoss-(gcss+deltag*kgss)-((1-mu)*(1/betaf-1)*omega*knoss*betaf-delta*koss+(1-1/betaf)-(1-rhof)*ofss/betaf+(1-div)*(1-tauo)*yoss);
tss=deltag*kgss+gcss+(1-mu)*iss*bss-(1-rhof)*ofss/betaf;

model(linear);
infl=1/(1+iss)*infl(+1)+(1-theta/(1+iss))*(1-theta)*(1-alpha)*phi/(theta*(1-alpha+alpha*eps))*n-(1-theta/(1+iss))*(1-theta)*(1-alpha)/(theta*(1-alpha+alpha*eps))*yno+(1-gamma)/gamma*((1-theta/(1+iss))*(1-theta)*(1-alpha)/(theta*(1-alpha+alpha*eps))+1+1/(1+iss))*rer-(1-gamma)/gamma*rer(-1)-(1-gamma)/(gamma*(1+iss))*rer(+1);
c=c(+1)+(mu*nss^phi*phi/css+(1-mu)*nss^phi/css)*n-((1-mu)*nss^phi/css+mu*nss^phi*phi/css)*n(+1)-(1-mu)*(css-nss^phi/phi)/(sigma*css)*(ir-infl(+1))+mu*tss/css*(tax(+1)-tax);
kappa*(1+1/(1+iss))*inv-kappa/(1+iss)*inv(+1)-kappa*inv(-1)=((1-delta)/(1+iss)+omega*(betaf-1/(1+iss)))*(kappa*(1+1/(1+iss))*inv(+1)-kappa/(1+iss)*inv(+2)-kappa*inv)+(1-(1-delta)/(1+iss)-omega*(betaf-1/(1+iss)))*(phi*n(+1)-kno)-(1-omega*(betaf-1/(1+iss)))*(ir-infl(+1))+omega*(betaf-1/(1+iss))*inff(+1)+omega*(betaf-1/(1+iss))*rer-omega*(betaf-1/(1+iss))*rer(+1)+omega*(betaf-1/(1+iss))*lagr;
ir=rho_ir*ir(-1)+(1-rho_ir)*(phiinf*(so*(po-po(-1))+(1-so)*infl+(so-1+gamma)/gamma*(rer-rer(-1)))+phiy*y);
nx=(bfy*(1-mu)-(1-rhof)*ofy-fxry*(1-mu))/nxy*iff(-1)+(bfy*(1-mu)/(nxy*betaf)-fdiy/nxy-fxry*(1-mu)/(nxy*betaf)-(1-rhof)*ofy/(nxy*betaf)+(1-div)*(1-tauo)*so/nxy)*rer+bfy*(1-mu)/(nxy*betaf)*bf(-1)+(fxry*(1-mu)-bfy*(1-mu))/(nxy*betaf)*rer(-1)+((1-rhof)*ofy-bfy*(1-mu)+fxry*(1-mu))/(nxy*betaf)*inff-bfy*(1-mu)/nxy*bf-ofy*(1-rhof)/(nxy*betaf)*of(-1)-fdiy/nxy*fdi+fxry*(1-mu)/nxy*fxr-fxry*(1-mu)/(nxy*betaf)*fxr(-1)+(1-div)*(1-tauo)*so/nxy*(yo+po);
iff=phiinff*inff+phiyf*cf;
inff=betaf*inff(+1)+(1-betaf*thetaf)*(1-thetaf)/thetaf*(sigma+(phif+alphaf)/(1-alphaf))*cf;
cf=rhouf*cf(-1)+ecf;
bf=inff(+1)-iff+kno+rer-rer(+1)+kappa*(1+1/(1+iss))*inv(+1)-kappa/(1+iss)*inv(+2)-kappa*inv;
kno=(1-delta)*kno(-1)+delta*inv;
inv=(y-cy*c-gcy*gc-giy*gi-(gcy+giy)*pg-nxy*nx)/((1-cy-gcy-giy-nxy)*(1-mu));
ko=(1-delta)*ko(-1)+delta*fdi;
kg=(1-deltag)*kg(-1)+deltag*gi;
gi=rhogi*gi(-1)+(1-rhogi)*(vgi*yo-gamgi*b(-1)+gamofgi*(of(-1)+iff(-1)+rer-inff));
gc=rhogc*gc(-1)+(1-rhogc)*(vgc*yo-gamgc*b(-1)+gamofgc*(of(-1)+iff(-1)+rer-inff));
tax=phib*b(-1)+phigi*gi+phigc*gc-phiof*(of(-1)+iff(-1)+rer-inff);
rer(+1)=ir+rer-betaf*(1+iss)*iff+inff(+1)-infl(+1)-(betaf*(1+iss)-1)*lagr;
fxr=rhofxr*fxr(-1)+(1-rhofxr)*(a1*rer+a2*(rer-rer(-1)));
b=(1+iss)*b(-1)+ir(-1)-(1+iss)*infl+giy/(by*(1-mu))*gi+gcy/(by*(1-mu))*gc+(gcy+giy)/(by*(1-mu))*pg-tss/(bss*(1-mu))*tax-(1-rhof)*ofy/(by*(1-mu))*iff(-1)+(1-rhof)*ofy/(betaf*by*(1-mu))*inff-(1-rhof)*ofy/(by*(1-mu)*betaf)*(rer+of(-1));
of=rhof/betaf*(of(-1)-inff)+so*(tauo+div*(1-tauo))/ofy*(yo+po)+rhof*iff(-1);
yo=alphao*ko(-1);
yno=alpha*kno(-1)+(1-alpha)*n+psi*kg(-1);
y=(1-so)*yno+so*yo+so*rer+(1-so)*ph+so*po;
pg=gamma2*ph+(1-gamma2)*rer;
ph+yno=gamma*cy/(1-so)*c+gamma*(1-cy-gcy-giy-nxy)*(1-mu)/(1-so)*inv+gamma2*gcy/(1-so)*gc+gamma2*giy/(1-so)*gi+gamma2*(gcy+giy)/(1-so)*pg;
infl=inflh+(1-gamma)/gamma*(rer-rer(-1));
c=((phi*nss^phi+(1-mu)*knoss*phi*(iss+delta-omega*(betaf-1/(1+iss))*(1+iss)))/css-(1-mu)*(1-so)*phi*(eps-1)/(eps*cy))*n-tss/css*tax+(1-mu)*(1+iss)*by/cy*(b(-1)-infl)+(1-mu)*by/cy*ir(-1)+(1-mu)/cy*(bfy*bf-(1-cy-gcy-giy-nxy)*inv-by*b)-(1-mu)*bfy/(cy*betaf)*bf(-1)+(fxry*(1-mu)/(cy*betaf)-(1-mu)*(1-so)*(eps-1)*(1-gamma)/(cy*eps*gamma)-(1-mu)*bfy/(cy*betaf))*rer+((1-mu)*bfy/(cy*betaf)-fxry*(1-mu)/(cy*betaf))*(rer(-1)+inff)+(fxry*(1-mu)/cy-(1-mu)*bfy/cy)*iff(-1)+(1-mu)*(1-so)/cy*(yno+ph)+fxry*(1-mu)/(cy*betaf)*fxr(-1)-fxry*(1-mu)/cy*fxr;
fdi=rhofdi*fdi(-1)+(1-rhofdi)*po;
po=rho_po*po(-1)-epo;
end;

% initialization
initval;
n=0;
y=0;
kg=0;
kno=0;
ko=0;
of=0;
fdi=0;
po=0;
yo=0;
yno=0;
c=0;
gc=0;
nx=0;
infl=0;
inff=0;
b=0;
rer=0;
ir=0;
cf=0;
iff=0;
bf=0;
fxr=0;
ph=0;
pg=0;
inv=0;
gi=0;
tax=0;
inflh=0;
lagr=0;
epo=0;
ecf=0;
end;

shocks;
var epo; stderr 0.1; 
//var ecf; stderr 0.1;
end;

resid;
steady(solve_algo=2,maxit=20000); 
check;

% IRFs
stoch_simul(irf=20,order=2) y, yno, yo, c, bf, gi, gc, infl, rer, b, ko, kno, kg, pg, n, po, fxr, nx;

